import os
import re
from difflib import SequenceMatcher, get_close_matches
from typing import Tuple, Union

import pandas as pd

_src_path = os.path.dirname(__file__)

_data = pd.read_feather(os.path.join(_src_path, "data/state.ft"))
_states = list(_data["State"])

_data = pd.read_feather(os.path.join(_src_path, "data/city.ft"))
_cities = list(_data["City"])

_data = pd.read_feather(os.path.join(_src_path, "data/address.ft"))
address = list(_data["Address"])

_data = pd.read_feather(os.path.join(_src_path, "data/state_and_city.ft"))
_states_and_cities = list(_data["StateAndCity"])

_state_expression = "(" + "|".join(_states) + ")"
_city_expression = "(" + "|".join(_cities) + ")"
_state_regex = re.compile("^" + _state_expression)
_city_regex = re.compile("^" + _city_expression)


def split_address(addr: str) -> Tuple[Union[str, None], Union[str, None], Union[str, None]]:
    m = _state_regex.match(addr)
    if not m:
        return None, None, None
    state = m[1]
    m = _city_regex.match(addr[len(state) :])
    if not m:
        return state, None, None
    city = m[1]
    return state, city, addr[len(state) + len(city) :]


def _state_complete(_target: str) -> Union[str, None]:
    return _target if _target in _states else None


def _city_complete(_target: str) -> Union[str, None]:
    matches = [c for c in _states_and_cities if re.search(_target + "$", c)]
    return None if len(matches) == 0 else matches[0]


def _complete(a: str, b: str) -> str:
    if len(a) <= 10:
        res = _state_complete(a)
        if res is not None:
            return res
        res = _city_complete(a)
        if res is not None:
            return res

    s = SequenceMatcher(None, a, b)
    res = ""
    for tag, i1, i2, j1, j2 in s.get_opcodes():
        if tag == "equal":
            res += a[i1:i2]
        elif tag == "replace":
            res += b[j1:j2]
        elif tag == "insert":
            res += b[j1:j2]
        elif tag == "delete" and len(b) == j1:  # pragma: no cover
            res += a[i1:i2]
        else:  # pragma: no cover
            pass
    return res


def _normalize(address: str) -> Tuple[str, str]:
    m = re.search("^(.+?)([0-9０-９]+.*)", address)
    if m is not None:
        return m.group(1), m.group(2)
    return address, ""


def _rule_based_complete(address: str) -> str:
    rules = [("東京都大手町", "東京都千代田区大手町")]
    for rule in rules:
        address = re.sub(rule[0], rule[1], address)
    return address


def complete_address(addr: str) -> str:
    addr = _rule_based_complete(addr)
    head, tail = _normalize(addr)
    candidates = get_close_matches(head, address, n=1, cutoff=0.3)
    complemented_head = _complete(head, candidates[0]) if len(candidates) > 0 else head
    return complemented_head + tail



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nayose']

package_data = \
{'': ['*'], 'nayose': ['data/*']}

install_requires = \
['feather-format>=0.4.0,<0.5.0', 'pandas>=1.0.1,<2.0.0']

setup_kwargs = {
    'name': 'nayose',
    'version': '0.1.11',
    'description': 'nayose is a Python module for data cleansing for Japanese address and is distributed under the MIT license.',
    'long_description': '\nnayose is a Python module for data cleansing for Japanese address and is distributed under the MIT license.\n\nInstallation\n============\n\nYou can install the latest this module with the command:\n\n    pip install nayose\n\nQuick start\n============\n\nnayose has two main function of data cleansing for Japanese address.\n\n`complete_address` is a function to complete dirty address. For example, the followings is a case of missing state(都道府県).\n\n    from nayose import complete_address\n\n    complete_address("千代田区大手町1−1−1大手町あいうえビル")\n    > "東京都千代田区大手町1−1−1大手町あいうえビル"\n\n\n`split_address` is a function to split complete address to state(都道府県), city(市区郡) and street(その他).\n\n    from nayose import split_address\n\n    split_address("東京都千代田区大手町1−1−1大手町あいうえビル")\n    > (\'東京都\', \'千代田区\', \'大手町1−1−1大手町あいうえビル\')\n\n\nImportant links\n============\n\n- Official source code repo: https://github.com/sonesuke/nayose\n- Download releases: https://pypi.org/project/nayose/\n- Issue tracker: https://github.com/sonesuke/nayose/issues\n\nCitation\n============\n\nIf you use nayose in a publication, we would appreciate citations: https://github.com/sonesuke/nayose\n\n\nFor contributors\n============\n\nYou can contribute it quickly by using docker image with the command.\n\n    git clone https://github.com/sonesuke/nayose.git\n    docker-compose run nayose bash\n\n\nFor test.\n\n    bash scripts/test\n\n\nIf you add any modules, please re-build docker image after \'poetry update\' .\n\n\n\n',
    'author': 'sonesuke',
    'author_email': 'iamsonesuke@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sonesuke/nayose',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.10,<4.0.0',
}


setup(**setup_kwargs)



import os
from typing import Dict, Tuple

import numpy as np
import pandas as pd

src_path = os.path.dirname(__file__)


def _to_dict(df: pd.DataFrame) -> Dict[str, int]:
    names = list(df["name"])
    counts = list(df["count"])
    return {names[i]: counts[i] for i in range(len(names))}


_first_df = pd.read_feather(os.path.join(src_path, "data/first_name.ft"))
_first_all_count = np.sum(_first_df["count"])
_first = _to_dict(_first_df)

_last_df = pd.read_feather(os.path.join(src_path, "data/last_name.ft"))
_last_all_count = np.sum(_last_df["count"])
_last = _to_dict(_last_df)


def _score(last: str, first: str) -> np.float64:
    first_count = _first[first] if first in _first.keys() else 0
    last_count = _last[last] if last in _last.keys() else 0
    return np.log((first_count + 1) / _first_all_count) + np.log((last_count + 1) / _last_all_count)


def split_name(name: str) -> Tuple[str, str]:
    candidates = [(name[:i], name[i:]) for i in range(1, len(name))]
    candidates += [(name[i:], name[:i]) for i in range(1, len(name))]
    scores = [_score(x, y) for x, y in candidates]
    if len(scores) == 0:
        return name, ""
    candidate = candidates[np.argmax(scores)]
    return candidate[0], candidate[1]
